"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const utils_1 = require("./utils");
const path_1 = require("path");
const fs_1 = require("fs");
const os_1 = require("os");
const child_process_1 = require("child_process");
class NeoDB {
    constructor(httpPort = 7474, boltPort = 7687, option = {}) {
        this.getServerFilesLocation = () => (0, path_1.join)((0, utils_1.getNeoOndemandHomeDir)(), "versions", `neo4j-community-${this.version}`);
        this.getInstanceDataLocation = () => {
            const dir = (0, path_1.join)((0, utils_1.getNeoOndemandHomeDir)(), "instance-data", String(this.httpPort));
            if (!(0, fs_1.existsSync)(dir))
                (0, fs_1.mkdirSync)(dir, { recursive: true });
            return dir;
        };
        this.getCustomConfFileLocation = () => (0, path_1.join)(this.getInstanceDataLocation(), "conf", "neo4j.conf");
        this.getServerBinFile = () => (0, path_1.join)(this.getInstanceDataLocation(), "bin", (0, os_1.platform)() === "win32" ? "neo4j.bat" : "neo4j");
        this.setProperties = () => __awaiter(this, void 0, void 0, function* () {
            const customConfig = {
                "dbms.default_database": `neo-ondemand-${this.httpPort}`,
                "dbms.connector.https.enabled": false,
                "dbms.security.auth_enabled": false,
                "dbms.connector.bolt.listen_address": `:${this.boltPort}`,
                "dbms.connector.http.listen_address": `:${this.httpPort}`,
                "dbms.directories.data": (0, path_1.join)(this.getInstanceDataLocation(), "data"),
            };
            return new Promise((resolve, reject) => {
                (0, fs_1.readFile)(this.getCustomConfFileLocation(), "utf8", (err, data) => {
                    if (err)
                        reject(err);
                    const keysToUpdate = Object.keys(customConfig);
                    const updatedLines = data
                        .split("\n")
                        .filter((line) => keysToUpdate.some((k) => !line.includes(k)));
                    keysToUpdate.forEach((k) => updatedLines.push(`${k}=${customConfig[k]}`));
                    const newData = updatedLines.join("\n");
                    (0, fs_1.writeFile)(this.getCustomConfFileLocation(), newData, "utf8", (err) => {
                        if (err)
                            reject(err);
                        else
                            resolve(null);
                    });
                });
            });
        });
        this.cleanUp = () => {
            (0, fs_1.rmSync)(this.getInstanceDataLocation(), { recursive: true, force: true });
        };
        this.start = () => __awaiter(this, void 0, void 0, function* () {
            const baseFiles = this.getServerFilesLocation();
            if (!(0, fs_1.existsSync)(baseFiles)) {
                yield (0, utils_1.downloadNeoCommunityEdition)(this.version);
                console.log("download complete");
            }
            (0, fs_1.cpSync)(baseFiles, this.getInstanceDataLocation(), { recursive: true });
            yield this.setProperties();
            const binFile = this.getServerBinFile();
            (0, fs_1.chmodSync)(binFile, "755");
            this.dbProcess = (0, child_process_1.spawn)(binFile, ["console"], {
                env: Object.assign(Object.assign({}, process.env), { NEO4J_CONF: (0, path_1.join)(this.getInstanceDataLocation(), "conf") }),
            });
            this.dbProcess.on("close", () => {
                if (!this.persistData)
                    this.cleanUp();
            });
            process.on("exit", () => {
                this.stop();
            });
            const dbProcess = this.dbProcess;
            return new Promise((resolve, reject) => {
                dbProcess.stdout.on("data", (message) => {
                    message = message.toString();
                    if (message.indexOf("Started.") !== -1) {
                        resolve(null);
                    }
                });
                dbProcess.stderr.on("data", (message) => {
                    message = message.toString();
                    console.error(message);
                });
                dbProcess.on("exit", () => {
                    reject(new Error(`${this.version} server could not be started`));
                });
            });
        });
        this.stop = () => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!this.persistData)
                this.cleanUp();
            (_a = this.dbProcess) === null || _a === void 0 ? void 0 : _a.kill();
            return new Promise((resolve) => {
                var _a;
                if (!this.dbProcess)
                    resolve(null);
                (_a = this.dbProcess) === null || _a === void 0 ? void 0 : _a.on("exit", () => resolve(null));
            });
        });
        this.getBoltURL = () => `bolt://localhost:${this.boltPort}`;
        this.getHttpURL = () => `http://localhost:${this.httpPort}`;
        this.httpPort = httpPort;
        this.boltPort = boltPort;
        this.version = option.version || "4.4.10";
        this.persistData = option.persistData || false;
    }
}
module.exports = NeoDB;
