"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadNeoCommunityEdition = exports.getNeoOndemandHomeDir = void 0;
const fs_1 = require("fs");
const https_1 = require("https");
const os_1 = require("os");
const path_1 = require("path");
const tar_1 = require("tar");
const AdmZip = require("adm-zip");
const getNeoOndemandHomeDir = () => {
    return (0, path_1.join)((0, os_1.homedir)(), ".neo-on-demand");
};
exports.getNeoOndemandHomeDir = getNeoOndemandHomeDir;
const _download = (url, targetFile) => __awaiter(void 0, void 0, void 0, function* () {
    const neoOnDemandHomeDir = getNeoOndemandHomeDir();
    const saveLocation = (0, path_1.join)(neoOnDemandHomeDir, "temp");
    const savePath = (0, path_1.join)(saveLocation, targetFile);
    if (!(0, fs_1.existsSync)(saveLocation))
        (0, fs_1.mkdirSync)(saveLocation, { recursive: true });
    return yield new Promise((resolve, reject) => {
        (0, https_1.get)(url, (response) => {
            var _a;
            const code = (_a = response.statusCode) !== null && _a !== void 0 ? _a : 0;
            if (code >= 400) {
                reject(new Error(response.statusMessage));
            }
            else if (code > 300 && code < 400 && !!response.headers.location) {
                _download(response.headers.location, targetFile).then(() => resolve(null));
            }
            else {
                const fileWriter = (0, fs_1.createWriteStream)(savePath);
                fileWriter.on("finish", () => {
                    fileWriter.close(() => resolve(null));
                });
                fileWriter.on("error", () => {
                    (0, fs_1.unlink)(savePath, () => reject(new Error("could not close fileWriter")));
                });
                response.pipe(fileWriter);
            }
        }).on("error", (error) => {
            reject(error);
        });
    });
});
const _extractZipFile = (zipPath, extractTo) => __awaiter(void 0, void 0, void 0, function* () {
    var zip = new AdmZip(zipPath);
    return new Promise((resolve, reject) => {
        zip.extractAllToAsync(extractTo, false, false, (err) => {
            if (err)
                reject(err);
            else
                resolve(null);
        });
    });
});
const _extractTarFile = (tarPath, extractTo) => __awaiter(void 0, void 0, void 0, function* () {
    if (!(0, fs_1.existsSync)(extractTo))
        (0, fs_1.mkdirSync)(extractTo, { recursive: true });
    return yield (0, tar_1.extract)({ file: tarPath, cwd: extractTo });
});
const _extractDBFiles = (filename, version) => __awaiter(void 0, void 0, void 0, function* () {
    const downloadedFilePath = (0, path_1.join)(getNeoOndemandHomeDir(), "temp", filename);
    const extractedPath = (0, path_1.join)(getNeoOndemandHomeDir(), "versions");
    if (!(0, fs_1.existsSync)(downloadedFilePath))
        throw new Error("File not found for extracting");
    if (filename.endsWith("zip")) {
        yield _extractZipFile(downloadedFilePath, extractedPath);
    }
    else {
        yield _extractTarFile(downloadedFilePath, extractedPath);
    }
    return new Promise((resolve) => (0, fs_1.rm)(downloadedFilePath, () => {
        resolve(null);
    }));
});
const downloadNeoCommunityEdition = (version) => __awaiter(void 0, void 0, void 0, function* () {
    let url, targetFile;
    if ((0, os_1.platform)() === "win32") {
        url = `https://neo4j.com/artifact.php?name=neo4j-community-${version}-windows.zip`;
        targetFile = `neo4j-community-${version}-windows.zip`;
    }
    else {
        url = `https://neo4j.com/artifact.php?name=neo4j-community-${version}-unix.tar.gz`;
        targetFile = `neo4j-community-${version}-unix.tar.gz`;
    }
    console.log(`downloading community edition version ${version}`);
    return _download(url, targetFile).then(() => _extractDBFiles(targetFile, version));
});
exports.downloadNeoCommunityEdition = downloadNeoCommunityEdition;
